// FamilyCode12Servlet.java - A demo servlet based on OneWireContainer10.
//
// Copyright (C) 1999-2002  Smart Software Consulting
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Smart Software Consulting
// 1688 Silverwood Court
// Danville, CA  94526-3079
// USA
//
// http://www.smartsc.com
//

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

import javax.servlet.*;
import javax.servlet.http.*;

import com.dalsemi.onewire.OneWireException;
import com.dalsemi.onewire.adapter.DSPortAdapter;
import com.dalsemi.onewire.container.OneWireContainer;
import com.dalsemi.onewire.container.OneWireContainer12;
import com.dalsemi.onewire.utils.Address;

public class FamilyCode12Servlet
extends HttpServlet implements SingleThreadModel
{
	private String a_channel;
	private String b_channel;

	public void doGet( HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		// Pick up all the parameters from the requestor
		Enumeration values = req.getParameterNames();

        a_channel = "";
        b_channel = "";

		// Resolve the expected
		String adapter = req.getParameter( OneWireServlet.ADAPTER);
		String port = req.getParameter( OneWireServlet.PORT);
		String ibString = req.getParameter( OneWireServlet.DEVICE);

		if( adapter  == null || port == null
		||  ibString == null || ibString.length() != 16)
		{
			throw new ServletException(
				getClass().getName() + ": Invalid parameters.");
		}

		byte[] iBId = Address.toByteArray( ibString);

		// Do HTTP header stuff
		res.setContentType( "text/html");

		// Get Writer
		PrintWriter pw = res.getWriter();

		// Start HTML
		pw.println( "<html>");
		pw.println( "<head><title>Dual Addressable Switch iButton Servlet</title></head>");
		pw.println( "<body>");

		// Identify any optional parameters
		while( values.hasMoreElements() )
        {
			String name, value;
			name = (String)values.nextElement();
			value = req.getParameter (name);
//          pw.println(name + " = " + value);
			if( name.equals("a_channel"))
			{
				a_channel = value.toLowerCase();
			}
			else if( name.equals("b_channel"))
			{
	    		b_channel = value.toLowerCase();
			}
		}

		boolean forecasting = false;
		boolean tabulating = false;

		DSPortAdapter pa = null;
		try
		{
			pa = OneWireServlet.getAdapter( adapter, port);
			OneWireContainer12 ib = new OneWireContainer12();
			ib.setupContainer( pa, iBId);
			sendBody( pw, pa, ib, adapter, port, ibString);
		}
		catch( Exception e)
		{
			OneWireServlet.sendAlert( pw, e);
			pw.println( "<P>Error: " + e.getMessage());
		}

		// Finish HTML
		pw.println("</body></html>");
	}

	public void sendBody(
		PrintWriter pw,
		DSPortAdapter pa,
		OneWireContainer12 ib,
		String adapter,
		String port,
		String ibutton)
	throws OneWireException
	{
		if( ib.isPresent())
		{
			pw.print( "<H1>Current Channel State of iButton ");
			pw.print( ib.getAddressAsString());
			pw.println( "</H1>");

			boolean bChannelA = false;
			boolean bChannelB = false;
			int iPrelude;

			ib.clearActivity();

			byte [] state = ib.readDevice();
			int iChannels = ib.getNumberChannels(state);
			boolean activity = ib.hasActivitySensing();
			boolean level = ib.hasLevelSensing();
			boolean smart = ib.hasSmartOn();

			pw.print("<p>This device has " + iChannels + " channel" + ((iChannels > 1) ? "s, " : ", "));
			pw.print((activity ? "has" : "does not have") + " activity sensing ability, ");
			pw.print((level ? "has" : "does not have") + " level sensing ability, and ");
			pw.print((smart ? "has" : "does not have") + " smart on ability.");

			if( a_channel.equals("toggle")) ib.setLatchState(0,!(ib.getLatchState(0,state)),smart,state);
			else if( a_channel.equals("enable")) ib.setLatchState(0,true,smart,state);
			else if( a_channel.equals("disable")) ib.setLatchState(0,false,smart,state);

			if( iChannels > 1)
			{
				if( b_channel.equals("toggle")) ib.setLatchState(1,!(ib.getLatchState(1,state)),smart,state);
				else if( b_channel.equals("enable")) ib.setLatchState(1,true,smart,state);
				else if( b_channel.equals("disable")) ib.setLatchState(1,false,smart,state);
			}

			try { Thread.sleep(500); }
			catch( InterruptedException ie) {} // Ignore

			ib.writeDevice(state);

			pw.println("<form action=\"FamilyCode12Servlet\">");
			pw.println("<p><font size=\"3\">Channel A: </font>");
			pw.println("<select name=\"a_channel\">");
			pw.println("<option value=\"default\"> </option>");
			pw.println("<option value=\"toggle\">Toggle</option>");
			pw.println("<option value=\"enable\">Enable</option>");
			pw.println("<option value=\"disable\">Disable</option>");
			pw.println("</select></p>");
			pw.println("<p><font size=\"3\">Channel B: </font>");
			pw.println("<select name=\"b_channel\">");
			pw.println("<option value=\"default\"> </option>");
			pw.println("<option value=\"toggle\">Toggle</option>");
			pw.println("<option value=\"enable\">Enable</option>");
			pw.println("<option value=\"disable\">Disable</option>");
			pw.println("</select></p>");
			pw.print  ("<input type=\"hidden\" name=\"");
			pw.print  (OneWireServlet.ADAPTER);
			pw.print  ("\" value=\"");
			pw.print  (adapter);
			pw.println("\"/>");
			pw.print  ("<input type=\"hidden\" name=\"");
			pw.print  (OneWireServlet.PORT);
			pw.print  ("\" value=\"");
			pw.print  (port);
			pw.println("\"/>");
			pw.print  ("<input type=\"hidden\" name=\"");
			pw.print  (OneWireServlet.DEVICE);
			pw.print  ("\" value=\"");
			pw.print  (ibutton);
			pw.println("\"/>");
			pw.println("<input type=\"submit\" name=\"Send\" value=\"Send\"/>");
			pw.println("</form>");

			state = ib.readDevice();
			bChannelA = ib.getLatchState(0,state);

			if (iChannels > 1)
			{
				state = ib.readDevice();
				bChannelB = ib.getLatchState(1,state);
			}

			pw.println( "<p>" + ((bChannelA) ? "A_ON" : "A_OFF") + " " + ((bChannelB) ? "B_ON" : "B_OFF") + "</p>");
		}
		else
		{
			pw.println( "<H1>iButton Not Found</H1>");
			pw.print( "iButton ");
			pw.print( ib.getAddressAsString());
			pw.print( " is not present on adapter '");
			pw.print( pa.getAdapterName());
			pw.print( "' on port '");
			pw.print( pa.getPortName());
			pw.println( "'.");
		}
	}
}
